@@ -1,5 +1,8 @@
 import sys
-import py, pytest
+
+import _pytest._code
+import py
+import pytest
 from _pytest.main import EXIT_NOTESTSCOLLECTED, EXIT_USAGEERROR
 
 
@@ -197,7 +200,7 @@ class TestGeneralUsage:
     def test_chdir(self, testdir):
         testdir.tmpdir.join("py").mksymlinkto(py._pydir)
         p = testdir.tmpdir.join("main.py")
-        p.write(py.code.Source("""
+        p.write(_pytest._code.Source("""
             import sys, os
             sys.path.insert(0, '')
             import py
@@ -450,19 +453,16 @@ class TestInvocationVariants:
             "*1 passed*",
         ])
 
-    @pytest.mark.skipif("sys.version_info < (2,5)")
     def test_python_minus_m_invocation_ok(self, testdir):
         p1 = testdir.makepyfile("def test_hello(): pass")
         res = testdir.run(py.std.sys.executable, "-m", "pytest", str(p1))
         assert res.ret == 0
 
-    @pytest.mark.skipif("sys.version_info < (2,5)")
     def test_python_minus_m_invocation_fail(self, testdir):
         p1 = testdir.makepyfile("def test_fail(): 0/0")
         res = testdir.run(py.std.sys.executable, "-m", "pytest", str(p1))
         assert res.ret == 1
 
-    @pytest.mark.skipif("sys.version_info < (2,5)")
     def test_python_pytest_package(self, testdir):
         p1 = testdir.makepyfile("def test_pass(): pass")
         res = testdir.run(py.std.sys.executable, "-m", "pytest", str(p1))
