@@ -1,6 +1,9 @@
 import sys
 from textwrap import dedent
-import pytest, py
+
+import _pytest._code
+import py
+import pytest
 from _pytest.main import EXIT_NOTESTSCOLLECTED
 
 
@@ -494,6 +497,7 @@ class TestFunction:
                 return True
         config.pluginmanager.register(MyPlugin1())
         config.pluginmanager.register(MyPlugin2())
+        config.hook.pytest_runtest_setup(item=item)
         config.hook.pytest_pyfunc_call(pyfuncitem=item)
 
     def test_multiple_parametrize(self, testdir):
@@ -527,6 +531,84 @@ class TestFunction:
         assert colitems[2].name == 'test2[a-c]'
         assert colitems[3].name == 'test2[b-c]'
 
+    def test_parametrize_skipif(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+
+            m = pytest.mark.skipif('True')
+
+            @pytest.mark.parametrize('x', [0, 1, m(2)])
+            def test_skip_if(x):
+                assert x < 2
+        """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines('* 2 passed, 1 skipped in *')
+
+    def test_parametrize_skip(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+
+            m = pytest.mark.skip('')
+
+            @pytest.mark.parametrize('x', [0, 1, m(2)])
+            def test_skip(x):
+                assert x < 2
+        """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines('* 2 passed, 1 skipped in *')
+
+    def test_parametrize_skipif_no_skip(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+
+            m = pytest.mark.skipif('False')
+
+            @pytest.mark.parametrize('x', [0, 1, m(2)])
+            def test_skipif_no_skip(x):
+                assert x < 2
+        """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines('* 1 failed, 2 passed in *')
+
+    def test_parametrize_xfail(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+
+            m = pytest.mark.xfail('True')
+
+            @pytest.mark.parametrize('x', [0, 1, m(2)])
+            def test_xfail(x):
+                assert x < 2
+        """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines('* 2 passed, 1 xfailed in *')
+
+    def test_parametrize_passed(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+
+            m = pytest.mark.xfail('True')
+
+            @pytest.mark.parametrize('x', [0, 1, m(2)])
+            def test_xfail(x):
+                pass
+        """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines('* 2 passed, 1 xpassed in *')
+
+    def test_parametrize_xfail_passed(self, testdir):
+        testdir.makepyfile("""
+            import pytest
+
+            m = pytest.mark.xfail('False')
+
+            @pytest.mark.parametrize('x', [0, 1, m(2)])
+            def test_passed(x):
+                pass
+        """)
+        result = testdir.runpytest()
+        result.stdout.fnmatch_lines('* 3 passed in *')
+
 
 class TestSorting:
     def test_check_equality(self, testdir):
@@ -598,13 +680,13 @@ class TestConftestCustomization:
 
     def test_customized_pymakemodule_issue205_subdir(self, testdir):
         b = testdir.mkdir("a").mkdir("b")
-        b.join("conftest.py").write(py.code.Source("""
+        b.join("conftest.py").write(_pytest._code.Source("""
             def pytest_pycollect_makemodule(__multicall__):
                 mod = __multicall__.execute()
                 mod.obj.hello = "world"
                 return mod
         """))
-        b.join("test_module.py").write(py.code.Source("""
+        b.join("test_module.py").write(_pytest._code.Source("""
             def test_hello():
                 assert hello == "world"
         """))
@@ -613,7 +695,7 @@ class TestConftestCustomization:
 
     def test_customized_pymakeitem(self, testdir):
         b = testdir.mkdir("a").mkdir("b")
-        b.join("conftest.py").write(py.code.Source("""
+        b.join("conftest.py").write(_pytest._code.Source("""
             import pytest
             @pytest.hookimpl(hookwrapper=True)
             def pytest_pycollect_makeitem():
@@ -624,7 +706,7 @@ class TestConftestCustomization:
                         for func in result:
                             func._some123 = "world"
         """))
-        b.join("test_module.py").write(py.code.Source("""
+        b.join("test_module.py").write(_pytest._code.Source("""
             import pytest
 
             @pytest.fixture()
@@ -662,7 +744,7 @@ class TestConftestCustomization:
 def test_setup_only_available_in_subdir(testdir):
     sub1 = testdir.mkpydir("sub1")
     sub2 = testdir.mkpydir("sub2")
-    sub1.join("conftest.py").write(py.code.Source("""
+    sub1.join("conftest.py").write(_pytest._code.Source("""
         import pytest
         def pytest_runtest_setup(item):
             assert item.fspath.purebasename == "test_in_sub1"
@@ -671,7 +753,7 @@ def test_setup_only_available_in_subdir(testdir):
         def pytest_runtest_teardown(item):
             assert item.fspath.purebasename == "test_in_sub1"
     """))
-    sub2.join("conftest.py").write(py.code.Source("""
+    sub2.join("conftest.py").write(_pytest._code.Source("""
         import pytest
         def pytest_runtest_setup(item):
             assert item.fspath.purebasename == "test_in_sub2"
@@ -787,7 +869,7 @@ class TestTracebackCutting:
         except ValueError:
             _, _, tb = sys.exc_info()
 
-        tb = py.code.Traceback(tb)
+        tb = _pytest._code.Traceback(tb)
         assert isinstance(tb[-1].path, str)
         assert not filter_traceback(tb[-1])
 
@@ -810,7 +892,7 @@ class TestTracebackCutting:
             _, _, tb = sys.exc_info()
 
         testdir.tmpdir.join('filter_traceback_entry_as_str.py').remove()
-        tb = py.code.Traceback(tb)
+        tb = _pytest._code.Traceback(tb)
         assert isinstance(tb[-1].path, str)
         assert filter_traceback(tb[-1])
 
