@@ -1,3 +1,5 @@
+import os
+
 import py, pytest
 from _pytest.mark import MarkGenerator as Mark
 
@@ -84,6 +86,22 @@ class TestMark:
         assert g.some.kwargs['reason2'] == "456"
 
 
+def test_marked_class_run_twice(testdir, request):
+    """Test fails file is run twice that contains marked class.
+    See issue#683.
+    """
+    py_file = testdir.makepyfile("""
+    import pytest
+    @pytest.mark.parametrize('abc', [1, 2, 3])
+    class Test1(object):
+        def test_1(self, abc):
+            assert abc in [1, 2, 3]
+    """)
+    file_name = os.path.basename(py_file.strpath)
+    rec = testdir.inline_run(file_name, file_name)
+    rec.assertoutcome(passed=6)
+
+
 def test_ini_markers(testdir):
     testdir.makeini("""
         [pytest]
@@ -298,7 +316,6 @@ class TestFunctional:
         assert 'hello' in keywords
         assert 'world' in keywords
 
-    @pytest.mark.skipif("sys.version_info < (2,6)")
     def test_mark_per_class_decorator(self, testdir):
         item = testdir.getitem("""
             import pytest
@@ -310,7 +327,6 @@ class TestFunctional:
         keywords = item.keywords
         assert 'hello' in keywords
 
-    @pytest.mark.skipif("sys.version_info < (2,6)")
     def test_mark_per_class_decorator_plus_existing_dec(self, testdir):
         item = testdir.getitem("""
             import pytest
@@ -565,7 +581,7 @@ class TestKeywordSelection:
         """)
         testdir.makepyfile(conftest="""
             import pytest
-            @pytest.mark.hookwrapper
+            @pytest.hookimpl(hookwrapper=True)
             def pytest_pycollect_makeitem(name):
                 outcome = yield
                 if name == "TestClass":
